"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebRollupConfig = void 0;
const plugin_commonjs_1 = __importDefault(require("@rollup/plugin-commonjs"));
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_replace_1 = __importDefault(require("@rollup/plugin-replace"));
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const plugin_json_1 = __importDefault(require("@rollup/plugin-json"));
const rollup_plugin_typescript2_1 = __importDefault(require("rollup-plugin-typescript2"));
const rollup_plugin_sass_1 = __importDefault(require("rollup-plugin-sass"));
const postcss_1 = __importDefault(require("postcss"));
const sass_1 = __importDefault(require("sass"));
const node_sass_magic_importer_1 = __importDefault(require("node-sass-magic-importer"));
const path_1 = require("path");
const typescript_1 = __importDefault(require("typescript"));
const rollup_plugin_svg_sprite_1 = __importDefault(require("./rollup-plugin-svg-sprite"));
const postcss_config_1 = __importDefault(require("./postcss.config"));
const isProd = process.env.NODE_ENV === 'production';
const extensions = ['.ts', '.tsx', '.js', '.jsx', '.es6', '.es', '.mjs'];
const WATCH_OPTS = {
    exclude: 'node_modules/**',
};
const cwd = process.cwd();
/**
 * Either a function that takes an id and returns true (external) or false (not external), or an Array of module IDs that should remain external to the bundle.
 * The IDs should be either:
 * 1) the name of an external dependency, exactly the way it is written in the import statement. I.e. to mark import "dependency.js" as external, use "dependency.js" while to mark import "dependency" as external, use "dependency".
 * 2) a resolved ID (like an absolute path to a file).
 * @link https://rollupjs.org/guide/en/#core-functionality
 */
const externalAll = (id, parentId, isResolved) => {
    // this means it is inside this package; not external
    if (id.startsWith('.'))
        return false;
    if ((0, path_1.isAbsolute)(id)) {
        if (id.includes('node_modules'))
            return true;
        // it's inside here
        if (id.startsWith(cwd))
            return false;
        if ((0, path_1.relative)(cwd, id).startsWith('..')) {
            console.error({ id, parentId, isResolved });
            throw new Error(`should not include something outside this directory`);
        }
    }
    // if not, then it's a dependency
    return true;
};
function createWebRollupConfig({ inputFile, distDir = './dist', testTypes, }) {
    return {
        input: inputFile,
        output: {
            format: 'es',
            file: (0, path_1.join)(distDir, 
            // if `inputFile` was `./src/index.ts`, then this would be `index.js`
            `${(0, path_1.parse)(inputFile).name}.js`),
            sourcemap: true,
        },
        external: externalAll,
        plugins: [
            (0, plugin_replace_1.default)({
                preventAssignment: true,
                values: {
                    'process.env.NODE_ENV': JSON.stringify(isProd ? 'production' : 'development'),
                },
            }),
            (0, plugin_node_resolve_1.default)({
                extensions,
                // exportConditions
                mainFields: ['module', 'main'],
                browser: true,
            }),
            (0, plugin_commonjs_1.default)({}),
            (0, rollup_plugin_typescript2_1.default)({
                typescript: typescript_1.default,
                useTsconfigDeclarationDir: false,
                check: testTypes,
                tsconfigOverride: {
                    compilerOptions: {
                        module: 'ESNext',
                    },
                },
            }),
            // babel({
            //   babelHelpers: 'runtime',
            //   extensions: ['.ts', '.tsx', '.mjs', '.cjs', '.js', '.jsx'],
            //   ...esBabelConfig,
            // }),
            (0, rollup_plugin_sass_1.default)({
                output: true,
                runtime: sass_1.default,
                options: {
                    includePaths: [(0, path_1.join)(process.cwd(), './node_modules/')],
                    outputStyle: 'compressed',
                    sourceMap: true,
                    importer: [(0, node_sass_magic_importer_1.default)()],
                },
                processor: async (styles, id) => {
                    return (0, postcss_1.default)(postcss_config_1.default.plugins).process(styles, {
                        from: id,
                    });
                },
            }),
            (0, plugin_json_1.default)({ indent: '  ', preferConst: true }),
            (0, rollup_plugin_svg_sprite_1.default)({
                outputFolder: distDir,
            }),
            isProd && (0, rollup_plugin_terser_1.terser)(),
        ],
        onwarn(warning, warn) {
            // https://github.com/rollup/rollup/blob/0fa9758cb7b1976537ae0875d085669e3a21e918/src/utils/error.ts#L324
            if (warning.code === 'UNRESOLVED_IMPORT') {
                console.log(`Failed to resolve the module ${warning.source} imported by ${warning.importer}` +
                    `\nIs the module installed? Note:` +
                    `\n ↳ to inline a module into your bundle, install it to "devDependencies".` +
                    `\n ↳ to depend on a module via import/require, install it to "dependencies".`);
                return;
            }
            warn(warning);
        },
    };
}
exports.createWebRollupConfig = createWebRollupConfig;
//# sourceMappingURL=rollup-config.js.map